import os
from .exceptions import HuskError
from .constants import (HUSK_CONTROL_DIR, HUSK_CONFIG_NAME,
    HUSK_NOTE_LOG_NAME)
from .config import Config
from .note import Notes

__all__ = ('Repo',)


class Repo(object):
    def __init__(self, path):
        self.path = os.path.abspath(path.rstrip('/'))
        self.config = Config(os.path.join(self.controldir, HUSK_CONFIG_NAME))
        self.notes = Notes(os.path.join(self.controldir,
            HUSK_NOTE_LOG_NAME), extension=self.config.get('general', 'extension'))

    @classmethod
    def isrepo(cls, path):
        "Checks if a `path` is an existing Husk repo."
        return os.path.exists(os.path.join(path, HUSK_CONTROL_DIR))

    @classmethod
    def findrepo(cls, path=None):
        path = path or os.getcwd()
        while True:
            if Repo.isrepo(path):
                return Repo(path)
            # No where else to go, break the loop
            if path == '/':
                raise HuskError('Repo does not exist in current directory ' \
                    'or any parent directory.')
            path = os.path.dirname(path)

    @classmethod
    def init(cls, path, defaults=False):
        "Shorthand method for initializing and writing to disk."
        # Ensure this is not an existing repo
        if cls.isrepo(path):
            raise HuskError('{} is already a Husk repository.'.format(path))

        repo = cls(path)
        repo.todisk(defaults)
        return repo

    @property
    def controldir(self):
        return os.path.join(self.path, HUSK_CONTROL_DIR)

    def relpath(self, path):
        "Returns a path relative to this repo given `path`."
        return os.path.relpath(os.path.join(os.getcwd(), path), self.path)

    def ondisk(self):
        "Returns whether this repo exists on disk."
        return os.path.exists(self.controldir)

    def todisk(self, defaults=False):
        "Writes the repo to disk."
        if not self.ondisk():
            # Make all directories up the control directory
            os.makedirs(self.controldir)

            if defaults:
                Config.write_defaults(self.config.path)
            self.notes.todisk()



import os
from ConfigParser import ConfigParser, NoOptionError
from .constants import HUSK_DEFAULT_CONFIG, HUSK_USER_CONFIG
from .exceptions import HuskError, HuskConfigError
from .decorators import cached_property

__all__ = ('Config', 'default_config')

noop = lambda x: x

# Map of config attributes and their intended types
config_types = {
    'notifications': {
        'enabled': bool,
        'days': int,
        'weeks': int,
        'years': int,
    }
}


class Config(object):
    """The Husk configuration class.

    If the init `path` exists and can be parsed, the settings will be
    read in and used.
    """
    def __init__(self, path=None):
        if isinstance(path, basestring):
            path = path.rstrip('/')
        self.path = path

    @classmethod
    def write_defaults(cls, path):
        "Write a copy of the default config to the specified file."
        if os.path.exists(path):
            raise HuskError('Cannot write default config, file already exists.')
        copy = open(path, 'w')
        default = open(HUSK_DEFAULT_CONFIG)
        copy.write(default.read())
        default.close()
        copy.close()

    @cached_property
    def parser(self):
        parser = ConfigParser()
        # Right-most takes precedence..
        parser.read([HUSK_DEFAULT_CONFIG, HUSK_USER_CONFIG, self.path])
        return parser

    def ondisk(self):
        return os.path.exists(self.path)

    def get(self, section, key, default=None):
        types = config_types.get(section, {})
        if self.parser.has_section(section):
            try:
                value = self.parser.get(section, key)
                return types.get(key, noop)(value)
            except NoOptionError:
                pass
            except (TypeError, ValueError):
                raise HuskConfigError('Invalid value in the "{}" section. ' \
                    'Parameter "{}" should be type {}'.format(section, key,
                        types[key].__name__))
        return default

    def set(self, section, key, value):
        "Set a config setting."
        if self.path is None:
            raise HuskError('Cannot set config. No repo config defined.')
        if not self.parser.has_section(section):
            self.parser.add_section(section)
        self.parser.set(section, key, value)

    def unset(self, section, key):
        "Unset a config setting."
        if self.path is None:
            raise HuskError('Cannot unset config. No repo config defined.')
        if self.parser.has_section(section):
            return self.parser.remove_option(section, key)
        return False

    def todisk(self):
        if self.path is None:
            raise HuskError('Cannot write to disk. No repo config defined.')
        with open(self.path) as config:
            self.parser.write(config)


def default_config():
    return Config()



import os
import sys
from cStringIO import StringIO
import shutil
import unittest
from husk import HuskError
from husk.commands import init, add, remove, move, info


class CLIInitTestCase(unittest.TestCase):
    def tearDown(self):
        shutil.rmtree('.husk')

    def test_init_noargs(self):
        options = init.parser.parse_args(''.split())
        init.parser.handle_raw(options)
        self.assertTrue(os.path.exists('.husk'))
        self.assertFalse(os.path.exists('.husk/config'))

    def test_init_existing(self):
        options = init.parser.parse_args(''.split())
        init.parser.handle_raw(options)
        self.assertTrue(os.path.exists('.husk'))
        self.assertRaises(HuskError, init.parser.handle_raw, options)

    def test_init_path(self):
        options = init.parser.parse_args(os.getcwd().split())
        init.parser.handle_raw(options)
        self.assertTrue(os.path.exists('.husk'))

    def test_init_config(self):
        options = init.parser.parse_args('-d'.split())
        init.parser.handle_raw(options)
        self.assertTrue(os.path.exists('.husk/config'))


class CLIAddTestCase(unittest.TestCase):
    def test_add(self):
        options = init.parser.parse_args()
        init.parser.handle_raw(options)

        options = add.parser.parse_args('python/cli python/web'.split())
        add.parser.handle_raw(options)
        self.assertTrue(os.path.exists('python/cli/cues.md'))
        self.assertTrue(os.path.exists('python/web/summary.md'))
        shutil.rmtree('python')
        shutil.rmtree('.husk')


class CLIRemoveTestCase(unittest.TestCase):
    def test_remove(self):
        options = init.parser.parse_args()
        init.parser.handle_raw(options)

        options = add.parser.parse_args('python/cli python/web'.split())
        add.parser.handle_raw(options)

        options = remove.parser.parse_args('python/cli'.split())
        remove.parser.handle_raw(options)
        self.assertTrue(os.path.exists('python/cli'))

        options = remove.parser.parse_args('python/web -d'.split())
        remove.parser.handle_raw(options)
        self.assertFalse(os.path.exists('python/web'))

        shutil.rmtree('.husk')


class CLIMoveTestCase(unittest.TestCase):
    def test_move(self):
        options = init.parser.parse_args()
        init.parser.handle_raw(options)

        options = add.parser.parse_args('python/cli python/web'.split())
        add.parser.handle_raw(options)

        options = move.parser.parse_args('python/cli python/command-line'.split())
        move.parser.handle_raw(options)
        self.assertTrue(os.path.exists('python/command-line'))
        self.assertFalse(os.path.exists('python/cli'))

        shutil.rmtree('.husk')
        shutil.rmtree('python')


class CLIInfoTestCase(unittest.TestCase):
    def test_info(self):
        options = init.parser.parse_args()
        init.parser.handle_raw(options)

        options = add.parser.parse_args('python/cli python/web'.split())
        add.parser.handle_raw(options)

        options = info.parser.parse_args([])
        stdout = sys.stdout
        sys.stdout = StringIO()
        info.parser.handle_raw(options)
        self.assertEqual(sys.stdout.getvalue(), '2 notes\n-----\npython/cli\npython/web\n')
        sys.stdout = stdout

        shutil.rmtree('.husk')
        shutil.rmtree('python')


if __name__ == '__main__':
    unittest.main()



from os import path, environ

__all__ = ('HUSK_CONTROL_DIR', 'HUSK_CONFIG_NAME', 'HUSK_NOTE_LOG_NAME',
    'HUSK_DEFAULT_CONFIG', 'HUSK_USER_CONFIG', 'HUSK_NOTE_FILES')

# Name of the directory that contains all the repo-specific files
# including log and configuration files
HUSK_CONTROL_DIR = '.husk'

HUSK_CONFIG_NAME = 'config'

# The `bundles` file is local to a repository and stores relative paths to
# all bundles in a repository.
HUSK_NOTE_LOG_NAME = 'notes'

# Absolute path to the default Husk config file
HUSK_DEFAULT_CONFIG = path.abspath(path.join(path.dirname(__file__),
    HUSK_CONFIG_NAME))

# Absolute path to the user-specific Husk config located in their
# home directory
HUSK_USER_CONFIG = path.abspath(path.join(environ['HOME'], '.huskconfig'))

# Names of files that are created within the bundle
HUSK_NOTE_FILES = ('outline', 'cues', 'summary')



"Husk is an API and CLI written in Python for the Cornell Note-taking System."

from .exceptions import *
from .constants import *
from .config import *
from .note import *
from .repo import *

__version__ = '0.1.2'



from husk import Repo
from husk.decorators import cli

__doc__ = """\
Shows various info about the repo.
"""


@cli(description=__doc__)
def parser(options):
    # Find the nearest repo
    repo = Repo.findrepo(options.repo)

    if 'notes' in options.stats:
        print('{} notes'.format(len(repo.notes)))
        print('-' * 5)
        for key in repo.notes:
            print(repo.relpath(key))


parser.add_argument('stats', nargs='*', default='notes',
    choices=['notes'], help='Specify which kinds of info to be presented')
parser.add_argument('-r', '--repo', help='Path to Husk repository')



import os
import shlex
import subprocess
from husk import Repo, HuskError
from husk.decorators import cli

__doc__ = """\
Command to "work on" a note of files with the editor defined
in your Husk settings and falls back to what is set in the $EDITOR
environment variable.
"""


@cli(description=__doc__)
def parser(options):
    # Find the nearest repo
    repo = Repo.findrepo(options.repo)

    # Adjust path
    path = repo.relpath(options.path)

    # Ensure the note exists before attempting to work on it
    if path not in repo.notes:
        raise HuskError('{} note does not exist'.format(path))

    editor = repo.config.get('general', 'editor', os.environ.get('EDITOR'))

    if not editor:
        raise HuskError('Cannot open notes. The "general.editor" ' \
            'setting nor the $EDITOR environment variable are defined.')

    args = shlex.split(editor) + repo.notes[path].files
    retcode = subprocess.call(args)

    if retcode != 0:
        raise HuskError('Editor had a non-successful exit. Nothing has been ' \
            'committed. To manually commit, use `husk commit`.')


parser.add_argument('path', help='Path to note directory')
parser.add_argument('files', nargs='*', help='Filenames to be opened')
parser.add_argument('-r', '--repo', help='Path to Husk repository')



import os
from collections import OrderedDict
from .config import default_config
from .constants import HUSK_NOTE_FILES
from .exceptions import HuskError

__all__ = ('Notes', 'Note')


class Notes(object):
    "dict-like container for managing multiple notes."
    def __init__(self, path, extension=None):
        if extension is None:
            extension = default_config().get('general', 'extension')
        self.path = path.rstrip('/')
        self.extension = extension

        self._notes = OrderedDict()

        # If this does exist, populate notes
        if self.ondisk():
            self.fromdisk()

    def __iter__(self):
        "Proxy to internal notes dict"
        for key in self._notes:
            yield key

    def __len__(self):
        "Proxy to internal notes dict"
        return len(self._notes)

    def __contains__(self, key):
        "Proxy to internal notes dict"
        if isinstance(key, basestring):
            key = key.rstrip('/')
        return key in self._notes

    def __getitem__(self, key):
        "Proxy to internal notes dict"
        if isinstance(key, basestring):
            key = key.rstrip('/')
        return self._notes[key]

    def add(self, note, defer=False):
        if not isinstance(note, Note):
            note = Note(note, self.extension)

        if note.path in self._notes:
            raise HuskError('{} note already exists in this ' \
                'repo.'.format(note.path))

        self._notes[note.path] = note

        # If on disk.. log immediately
        if self.ondisk():
            note.todisk()
            if not defer:
                with open(self.path, 'a') as log:
                    log.write('{}\n'.format(note.path))

    def remove(self, note, delete=False, defer=False):
        """Removes a note from the repo.

        If `rmdisk` is True, the note is removed from disk as well.
        If `defer` is True, the notelog will not be updated. This
        is useful to prevent redundant IO in a tight loop.
        """
        if not isinstance(note, Note):
            note = Note(note, self.extension)

        if note.path not in self._notes:
            raise HuskError('{} note does not exist in this ' \
                'repo.'.format(note.path))

        del self._notes[note.path]

        # If on disk.. log immediately
        if self.ondisk():
            if delete:
                note.rmdisk()
            if not defer:
                self.todisk()

    def move(self, note, target, defer=False):
        """Moves a note from it's current path to a different path."

        If `defer` is True, the notelog will not be updated. This
        is useful to prevent redundant IO in a tight loop.
        """
        if not isinstance(note, Note):
            note = Note(note, self.extension)

        if note.path not in self._notes:
            raise HuskError('{} note does not exist in this ' \
                'repo.'.format(note.path))

        # Ensure the target is not an existing note
        if target in self._notes:
            raise HuskError('{} already exists.'.format(target))

        # Reference old path
        source = note.path

        # If on disk.. perist to disk
        if self.ondisk():
            note.mvdisk(target)
            # Internally change after a succesful move occurred
            # to prevent inconsistencies
            del self._notes[source]
            self._notes[note.path] = note
            if not defer:
                self.todisk()
        else:
            note.path = target
            del self._notes[source]
            self._notes[target] = note

    def ondisk(self):
        return os.path.exists(self.path)

    def todisk(self):
        with open(self.path, 'w') as log:
            for name in self._notes:
                self._notes[name].todisk()
                log.write('{}\n'.format(name))

    def fromdisk(self, force=False):
        if len(self) and not force:
            raise HuskError('Fail to load from disk. Notes already exist. ' \
                'Pass `force=True` to override.')
        with open(self.path) as log:
            for line in log.xreadlines():
                path = line.strip()
                self._notes[path] = Note(path, self.extension)


class Note(object):
    def __init__(self, path, extension):
        self.path = path.rstrip('/')
        self.extension = extension

    def ondisk(self):
        return os.path.exists(self.path)

    def todisk(self):
        if not self.ondisk():
            os.makedirs(self.path)

        # Create empty files if they do not already exist
        for name in HUSK_NOTE_FILES:
            path = os.path.join(self.path, '{}.{}'.format(name, self.extension))
            if not os.path.exists(path):
                open(path, 'w').close()

    def rmdisk(self):
        # Remove known files
        for name in self.files:
            os.remove(name)

        # Attempt to remove all directories including leaf. OSError is
        # only raised if the leaf cannot be removed. Errors for attempting
        # to remove parent directories are ignored
        try:
            os.removedirs(self.path)
        except OSError:
            print('{} is not empty, not removing'.format(self.path))

    def mvdisk(self, target):
        if self.ondisk():
            os.renames(self.path, target)
        self.path = target

    @property
    def files(self, names=None):
        names = names or HUSK_NOTE_FILES
        # Create empty files if they do not already exist
        return [os.path.join(self.path, '{}.{}'.format(name, self.extension)) \
            for name in names if name in HUSK_NOTE_FILES]



from husk import Repo
from husk.decorators import cli

__doc__ = """\
Adds a note to the repository specified by it's `path`. If `path`
is an existing directory/note, pass `--force` to register it with
the repository. Existing files of the same name will not be overwritten.
"""


@cli(description=__doc__)
def parser(options):
    # Find the nearest repo
    repo = Repo.findrepo(options.repo)

    for path in options.path:
        # Ensure the path is defined relative to the repo
        repo.notes.add(repo.relpath(path), defer=True)
    repo.notes.todisk()


parser.add_argument('path', nargs='*', help='Paths to note directory')
parser.add_argument('-r', '--repo', help='Path to Husk repository')
parser.add_argument('-f', '--force', action='store_true',
    help='Force an existing unregistered note to be added to the repository')



from husk import Repo
from husk.decorators import cli

__doc__ = """\
Moves an existing note to a new path.
"""


@cli(description=__doc__)
def parser(options):
    # Find the nearest repo
    repo = Repo.findrepo(options.repo)

    src = repo.relpath(options.src)
    dest = repo.relpath(options.dest)
    repo.notes.move(src, dest)


parser.add_argument('src', help='Source path to note that will be moved')
parser.add_argument('dest', help='Destination path of the note')
parser.add_argument('-r', '--repo', help='Path to Husk repository')



from husk.repo import Repo
from husk.decorators import cli

__doc__ = """\
Initialize a Husk repository in the current working directory (cwd)
unless a `path` is specified.
"""


@cli(description=__doc__)
def parser(options):
    Repo.init(options.path, options.defaults)


parser.add_argument('path', nargs='?', default='.',
    help='Path to Husk repository')
parser.add_argument('-d', '--defaults', action='store_true',
    help='Add a copy of the Husk defaults to the repo control directory.')



from husk import Repo
from husk.decorators import cli

__doc__ = """\
Remove a note from the specified repo. By default note files
are not deleted.
"""


@cli(description=__doc__)
def parser(options):
    # Find the nearest repo
    repo = Repo.findrepo(options.repo)

    for path in options.path:
        # Ensure the path is defined relative to the repo
        repo.notes.remove(repo.relpath(path), defer=True,
            delete=options.delete)
    repo.notes.todisk()


parser.add_argument('path', nargs='*', help='Path to note directory')
parser.add_argument('-r', '--repo', help='Path to Husk repository')
parser.add_argument('-d', '--delete', action='store_true',
    help='Delete note files from disk')



import husk
from setuptools import setup, find_packages

kwargs = {
    'packages': find_packages(),
    'include_package_data': True,
    'test_suite': 'tests',
    'name': 'husk',
    'version': husk.__version__,
    'author': 'Byron Ruth, Patrick Henning',
    'author_email': 'b@devel.io',
    'description': husk.__doc__,
    'license': 'BSD',
    'keywords': 'note-taking Cornell',
    'url': 'https://bruth.github.com/husk/',
    'scripts': ['bin/husk'],
    'classifiers': [
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 2.7'
    ],
}

setup(**kwargs)
